#ifndef TDATANEBULA_HH
#define TDATANEBULA_HH

#include "TSystem.h"
#include "TROOT.h"
#include "TGraph2DErrors.h"
#include "TGraph.h"
#include "TPad.h"
#include "TCanvas.h"
#include "Buttons.h"
#include "TView.h"
#include "TMarker3DBox.h"
#include "TCONE.h"
#include "TNode.h"
#include "TGeometry.h"
#include "TList.h"

#include "TLinearFit.hh"

#include <iostream>
#include <vector>
#include <list>
#include <sstream>
#include <algorithm>
#include <cmath>

static const Double_t mneutron = 939.565;
static const Double_t vlight = 29.979;

class TDataNEBULA
{
public:
  TDataNEBULA(){;}
  ~TDataNEBULA(){;}

  bool operator==(const TDataNEBULA& data){
    if(this->id == data.id){
      return true;
    }else{
      return false;
    }
  };

  int id;
  double quraw, qdraw, turaw, tdraw;
  double quped, qdped, qucal, qdcal, qaveped, qavecal;
  double tucal, tdcal, tuslw, tdslw, tavecal, taveslw;
  double logquped, logqdped, dtraw, dtcal, dtslw, poscal, posslw, posx;
  double tucalt0, tdcalt0, tuslwt0, tdslwt0, tavecalt0, taveslwt0, tavecalt0flcor, taveslwt0flcor;
  double flightlength;

  double time;
  std::vector<double> pos,beta, p;
  double betaabs, gamma, pabs, energy, theta, phi;
};

bool position(std::vector<TDataNEBULA>& data_v, std::vector<const TArtNEBULAPlaPara*>& /*para_v*/, double position)
{
  for(int i=0; i<(int)data_v.size(); ++i){
    TDataNEBULA data = data_v[i];
    //    int id = data.id;
    //    const TArtNEBULAPlaPara* para = para_v[id-1];

    std::vector<double> pos = data.pos;

    if(!(abs(pos[0]) < position && abs(pos[1]) < position)){
      data_v.erase(data_v.begin()+i);
      --i;
    }
  }

  return true;
}

bool acceptance(std::vector<TDataNEBULA>& data_v, std::vector<const TArtNEBULAPlaPara*>& /*para_v*/, double theta)
{
  for(int i=0; i<(int)data_v.size(); ++i){
    TDataNEBULA data = data_v[i];
    //    int id = data.id;
    //    const TArtNEBULAPlaPara* para = para_v[id-1];

    if(!(data.theta < theta)){
      data_v.erase(data_v.begin()+i);
      --i;
    }
  }

  return true;
}

bool acceptance2(std::vector<TDataNEBULA>& data_v, std::vector<const TArtNEBULAPlaPara*>& /*para_v*/, double theta1, double theta2)
{
  for(int i=0; i<(int)data_v.size(); ++i){
    TDataNEBULA data = data_v[i];
    //    int id = data.id;
    //    const TArtNEBULAPlaPara* para = para_v[id-1];

    if(!(data.theta > theta1 && data.theta < theta2)){
      data_v.erase(data_v.begin()+i);
      --i;
    }
  }

  return true;
}

// mode  0:all veto, 1:first only, <0:others 
bool veto(std::vector<TDataNEBULA>& data_v, std::vector<const TArtNEBULAPlaPara*>& para_v, int mode)
{
  bool veto = false;
  bool veto2 = false;
  for(int i=0; i<(int)data_v.size(); ++i){
    TDataNEBULA data = data_v[i];
    int id = data.id;
    const TArtNEBULAPlaPara* para = para_v[id-1];

    if(mode == 0){
      if(para->GetSubLayer() == 0) veto = true;
    }else if(mode == 1){
      if(para->GetSubLayer() == 0){
	if(para->GetLayer() == 1) veto = true;
      }
    }else if(mode == 2){
      if(para->GetSubLayer() == 0){
	if(para->GetLayer() == 1){
	  veto = true;
	}else if(para->GetLayer() == 2){
	  veto2 = true;
	}
      }
    }else if(mode < 0){
      // VETO by NEUT
      if(para->GetLayer() == 1){
	if(para->GetSubLayer() != 0){
	  veto = true;
	}
      }
    }else{
      std::cout << "this veto mode: " << mode << " is not available" << std::endl;
    }
  }

  if(veto2){
    for(int i=0; i<(int)data_v.size(); ++i){
      TDataNEBULA data = data_v[i];
      int id = data.id;
      const TArtNEBULAPlaPara* para = para_v[id-1];

      if(para->GetLayer() == 2){
	data_v.erase(data_v.begin()+i);
	--i;
      }
    }
  }

  return veto;
}

bool vetolocal(std::vector<TDataNEBULA>& data_v, std::vector<const TArtNEBULAPlaPara*>& para_v)
{
  std::vector<TDataNEBULA> killer;
  for(int i=0; i<(int)data_v.size(); ++i){
    TDataNEBULA data = data_v[i];
    int id = data.id;
    const TArtNEBULAPlaPara* para = para_v[id-1];

    if(para->GetSubLayer() == 0){
      if(para->GetLayer() == 1){
	return true;
      }else if(para->GetLayer() == 2){
	killer.push_back(data);
      }
    }
  }

  // kill veto
  for(int i=0; i<(int)data_v.size(); ++i){
    TDataNEBULA data = data_v[i];
    int id = data.id;
    const TArtNEBULAPlaPara* para = para_v[id-1];

    if(para->GetSubLayer() == 0){ // VETO
      data_v.erase(data_v.begin()+i);
      --i;
    }
  }

  for(int i=0; i<(int)data_v.size(); ++i){
    TDataNEBULA data = data_v[i];
    int id = data.id;
    const TArtNEBULAPlaPara* para = para_v[id-1];
    if(para->GetLayer() == 2){
      for(int j=0; j<(int)killer.size(); ++j){
	double dx = data.pos[0]-killer[j].pos[0];
	double dy = data.pos[1]-killer[j].pos[1];
	double r = sqrt(dx*dx+dy*dy);
	if((r < 60) && 
	   (data.time - killer[j].time > 1) &&
	   (data.time - killer[j].time < 6)){
	  data_v.erase(data_v.begin()+i);
	  --i;
	  break;
        }
      }
    }
  }

  return false;
}

void threshold(std::vector<TDataNEBULA>& data_v, std::vector<const TArtNEBULAPlaPara*>& para_v, double neut, double veto)
{
  for(int i=0; i<(int)data_v.size(); ++i){
    TDataNEBULA data = data_v[i];
    int id = data.id;
    const TArtNEBULAPlaPara* para = para_v[id-1];

    double qavecal = data.qavecal;
    if(para->GetSubLayer() != 0){ // NEUT
      if(!(qavecal > neut)){
	data_v.erase(data_v.begin()+i);
	--i;
	continue;
      }
    }else{ // VETO
      if(!(qavecal > veto)){
	data_v.erase(data_v.begin()+i);
	--i;
	continue;
      }
    }
  }
}

void killveto(std::vector<TDataNEBULA>& data_v, std::vector<const TArtNEBULAPlaPara*>& para_v)
{
  for(int i=0; i<(int)data_v.size(); ++i){
    TDataNEBULA data = data_v[i];
    int id = data.id;
    const TArtNEBULAPlaPara* para = para_v[id-1];

    if(para->GetSubLayer() == 0){ // VETO
      data_v.erase(data_v.begin()+i);
      --i;
    }
  }
}

void hitmintime(std::vector<TDataNEBULA>& data_v, std::vector<const TArtNEBULAPlaPara*>& para_v)
{
  // kill veto
  for(int i=0; i<(int)data_v.size(); ++i){
    TDataNEBULA data = data_v[i];
    int id = data.id;
    const TArtNEBULAPlaPara* para = para_v[id-1];

    if(para->GetSubLayer() == 0){ // VETO
      data_v.erase(data_v.begin()+i);
      --i;
    }
  }

  if(data_v.size() != 0){
    Double_t tmin = data_v[0].time;
    while((int)data_v.size() > 1){
      Double_t t = data_v[1].time;

      if(tmin > t){
	tmin = t;
	data_v.erase(data_v.begin());
      }else{
	data_v.erase(data_v.begin()+1);
      }
    }
  }
}

void hitmintime2(std::vector<TDataNEBULA>& data_v, std::vector<const TArtNEBULAPlaPara*>& para_v)
{
  // refill
  std::vector<std::vector<TDataNEBULA> > data_v_layer(2);
  for(int i=0; i<(int)data_v.size(); ++i){
    TDataNEBULA data = data_v[i];
    int id = data.id;
    const TArtNEBULAPlaPara* para = para_v[id-1];
 
    if(para->GetSubLayer() == 0){ // VETO
      /* kill */
    }else if(para->GetLayer() == 1){ // 1st layer NEUT
      data_v_layer[0].push_back(data_v[i]);
    }else if(para->GetLayer() == 2){ // 2nd layer NEUT
      data_v_layer[1].push_back(data_v[i]);
    }
  }
 
  if(data_v_layer[0].size() != 0){
    Double_t tmin = data_v_layer[0][0].time;
    while((int)data_v_layer[0].size() > 1){
      Double_t t = data_v_layer[0][1].time;
 
      if(tmin > t){
 	tmin = t;
 	data_v_layer[0].erase(data_v_layer[0].begin());
      }else{
 	data_v_layer[0].erase(data_v_layer[0].begin()+1);
      }
    }
  }
 
  if(data_v_layer[1].size() != 0){
    Double_t tmin = data_v_layer[1][0].time;
    while((int)data_v_layer[1].size() > 1){
      Double_t t = data_v_layer[1][1].time;
 
      if(tmin > t){
 	tmin = t;
 	data_v_layer[1].erase(data_v_layer[1].begin());
      }else{
 	data_v_layer[1].erase(data_v_layer[1].begin()+1);
      }
    }
  }
 
  data_v.clear();
  if(data_v_layer[0].size() == 1){
    data_v.push_back(data_v_layer[0][0]);
  }
  if(data_v_layer[1].size() == 1){
    data_v.push_back(data_v_layer[1][0]);
  }
}

void hitmintime4(std::vector<TDataNEBULA>& data_v, std::vector<const TArtNEBULAPlaPara*>& para_v)
{
  // refill
  std::vector<std::vector<TDataNEBULA> > data_v_layer(4);
  for(int i=0; i<(int)data_v.size(); ++i){
    TDataNEBULA data = data_v[i];
    int id = data.id;
    const TArtNEBULAPlaPara* para = para_v[id-1];
 
    if(para->GetSubLayer() == 0){ // VETO
      if(para->GetLayer() == 1){ // 1st layer VETO
	data_v_layer[2].push_back(data_v[i]);
      }else if(para->GetLayer() == 2){ // 2nd layer VETO
	data_v_layer[3].push_back(data_v[i]);
      }
    }else if(para->GetLayer() == 1){ // 1st layer NEUT
      data_v_layer[0].push_back(data_v[i]);
    }else if(para->GetLayer() == 2){ // 2nd layer NEUT
      data_v_layer[1].push_back(data_v[i]);
    }
  }
 
  if(data_v_layer[0].size() != 0){
    Double_t tmin = data_v_layer[0][0].time;
    while((int)data_v_layer[0].size() > 1){
      Double_t t = data_v_layer[0][1].time;
 
      if(tmin > t){
 	tmin = t;
 	data_v_layer[0].erase(data_v_layer[0].begin());
      }else{
 	data_v_layer[0].erase(data_v_layer[0].begin()+1);
      }
    }
  }
 
  if(data_v_layer[1].size() != 0){
    Double_t tmin = data_v_layer[1][0].time;
    while((int)data_v_layer[1].size() > 1){
      Double_t t = data_v_layer[1][1].time;
 
      if(tmin > t){
 	tmin = t;
 	data_v_layer[1].erase(data_v_layer[1].begin());
      }else{
 	data_v_layer[1].erase(data_v_layer[1].begin()+1);
      }
    }
  }
 
  if(data_v_layer[2].size() != 0){
    Double_t tmin = data_v_layer[2][0].time;
    while((int)data_v_layer[2].size() > 1){
      Double_t t = data_v_layer[2][2].time;
 
      if(tmin > t){
 	tmin = t;
 	data_v_layer[2].erase(data_v_layer[2].begin());
      }else{
 	data_v_layer[2].erase(data_v_layer[2].begin()+1);
      }
    }
  }

  if(data_v_layer[3].size() != 0){
    Double_t tmin = data_v_layer[3][0].time;
    while((int)data_v_layer[3].size() > 1){
      Double_t t = data_v_layer[3][3].time;
 
      if(tmin > t){
 	tmin = t;
 	data_v_layer[3].erase(data_v_layer[3].begin());
      }else{
 	data_v_layer[3].erase(data_v_layer[3].begin()+1);
      }
    }
  }

  data_v.clear();
  if(data_v_layer[0].size() == 1){
    data_v.push_back(data_v_layer[0][0]);
  }
  if(data_v_layer[1].size() == 1){
    data_v.push_back(data_v_layer[1][0]);
  }
  if(data_v_layer[2].size() == 1){
    data_v.push_back(data_v_layer[2][0]);
  }
  if(data_v_layer[3].size() == 1){
    data_v.push_back(data_v_layer[3][0]);
  }
}

bool energy(std::vector<TDataNEBULA>& data_v, std::vector<const TArtNEBULAPlaPara*>& /*para_v*/, double emin, double emax)
{
  bool flag = false;
  for(int i=0; i<(int)data_v.size(); ++i){
    TDataNEBULA data = data_v[i];
    //    int id = data.id;
    //    const TArtNEBULAPlaPara* para = para_v[id-1];

    double energy = data.energy;
    //    if(energy > 190) flag = true;
    if(energy > emin && energy < emax) flag = true;
  }

  return flag;
}

bool energyfast(std::vector<TDataNEBULA>& data_v, std::vector<const TArtNEBULAPlaPara*>& /*para_v*/, double emin, double emax)
{
  //sort
  std::map<double, TDataNEBULA> data_v_sort;
  for(int i=0; i<(int)data_v.size(); ++i){
    TDataNEBULA data = data_v[i];
    data_v_sort.insert(std::pair<double, TDataNEBULA>(data.time, data));
  }

  std::map<double, TDataNEBULA>::iterator it = data_v_sort.begin();  
  double energy = (it->second).energy;
  if(energy > emin && energy < emax) return true;
  else return false ;
}

bool sortbytime(std::vector<TDataNEBULA>& data_v, std::vector<const TArtNEBULAPlaPara*>& /*para_v*/)
{
  //sort
  std::map<double, TDataNEBULA> data_v_sort;
  for(int i=0; i<(int)data_v.size(); ++i){
    TDataNEBULA data = data_v[i];
    data_v_sort.insert(std::pair<double, TDataNEBULA>(data.time, data));
  }

  std::map<double, TDataNEBULA>::iterator it = data_v_sort.begin();  
  data_v.clear();
  while(it != data_v_sort.end()){
    data_v.push_back(it->second);
    ++it;
  }

  return true;
}

bool searchpknockout(std::vector<TDataNEBULA>& data_v, std::vector<const TArtNEBULAPlaPara*>& para_v)
{
  bool veto1 = false;
  bool veto2 = false;
  for(int i=0; i<(int)data_v.size(); ++i){
    TDataNEBULA data = data_v[i];
    int id = data.id;
    const TArtNEBULAPlaPara* para = para_v[id-1];

    if(para->GetSubLayer() == 0){
      if(para->GetLayer() == 1) veto1 = true;
      else if(para->GetLayer() == 2) veto2 = true;
    }
  }
  if(veto1 == true || veto2 == false) return false;

  hitmintime4(data_v, para_v);

  if(data_v.size() != 3) return false;

  std::vector<double> time(3);
  for(int i=0; i<(int)data_v.size(); ++i){
    TDataNEBULA data = data_v[i];
    int id = data.id;
    const TArtNEBULAPlaPara* para = para_v[id-1];

    if(para->GetSubLayer() == 0){
      time[1] = data_v[i].time;
    }else{
      if(para->GetLayer() == 1){
	time[0] = data_v[i].time;
      }else{
	time[2] = data_v[i].time;
      }
    }
  }

  if(!((time[0] < time[1]) &&
       (time[1] < time[2]))) return false;

  return true;
}

void cluster(std::vector<TDataNEBULA>& data_v, std::vector<const TArtNEBULAPlaPara*>& /*para_v*/)
{
  std::vector<std::vector<TDataNEBULA> > data_cluster_v;
  std::vector<TDataNEBULA> data_v_copy(data_v);

  std::vector<TDataNEBULA>::iterator it_newclusterhead  = data_v_copy.begin();
  while(it_newclusterhead != data_v_copy.end()){

    //既にdata_cluster_vの中にいないか探索
    bool found = false;
    std::vector<std::vector<TDataNEBULA> >::iterator it_data_cluster_v = data_cluster_v.begin();
    while(it_data_cluster_v != data_cluster_v.end()){
      if(std::find(it_data_cluster_v->begin(), it_data_cluster_v->end(), *it_newclusterhead) != it_data_cluster_v->end()){
	found = true;
      }
      ++it_data_cluster_v;
    }
    if(found){
      it_newclusterhead = data_v_copy.erase(it_newclusterhead);
      continue;
    }

    //    std::cout << "in b" << std::endl;

    //新しいclusterにdataを詰めていく
    std::list<TDataNEBULA> data_cluster_list;
    data_cluster_list.push_back(*it_newclusterhead);

    std::list<TDataNEBULA>::iterator it_cluster  = data_cluster_list.begin();
    while(it_cluster != data_cluster_list.end()){
      //      std::cout << "in c " << data_cluster_v.size() << " " << data_cluster_list.size() << std::endl;
      std::vector<TDataNEBULA>::iterator it_data = data_v_copy.begin();
      while(it_data != data_v_copy.end()){
	//	std::cout << "in d " << data_v_copy.size() << " " << data_cluster_list.size() << std::endl;
	if(std::find(data_cluster_list.begin(), data_cluster_list.end(), *it_data) != data_cluster_list.end()){ //既にclusterの中にいないか探索
	  ++it_data;
	  continue;
	}
	
	double dx = (it_cluster->pos[0]-it_data->pos[0])/24; // 割る24は近傍とみなす距離で規格化
	double dy = (it_cluster->pos[1]-it_data->pos[1])/24;
	double dz = (it_cluster->pos[2]-it_data->pos[2])/24;

	double dt = (it_cluster->time-it_data->time)/(24/vlight/0.6); //24cm飛ぶのにかかる時間で規格化 beta=0.6決め打ち
	double distance = sqrt((dx*dx+dy*dy+dz*dz+dt*dt)/4);
	//	std::cout << dx << " " << dy << " " << dz << " " << dt << " " << distance << std::endl;
	//	if(distance < 1) data_cluster_list.push_back(*it_data);
	if(distance < 1) data_cluster_list.push_back(*it_data);
	++it_data;
      }
      ++it_cluster;
    }
    std::vector<TDataNEBULA> data_cluster(data_cluster_list.size());
    std::copy(data_cluster_list.begin(), data_cluster_list.end(), data_cluster.begin());
    data_cluster_v.push_back(data_cluster);
    it_newclusterhead = data_v_copy.erase(it_newclusterhead);
  }

  //  std::cout << data_cluster_v.size() << std::endl;

  std::vector<TDataNEBULA> data_min_v;
  for(int i=0; i<(int)data_cluster_v.size(); ++i){
    std::vector<TDataNEBULA> data_v_copy = data_cluster_v[i];

    double tmin = 1e10;
    double jmin = 0;
    for(int j=0; j<(int)data_v_copy.size(); ++j){
      TDataNEBULA data = data_v_copy[j];
      double t = data.time;
      if(tmin > t){
	tmin = t;
	jmin = j;
      }
    }
    data_min_v.push_back(data_v_copy[jmin]);
  }
  
  data_v = data_min_v;
}

void visualize(std::vector<TDataNEBULA>& data_v, std::vector<const TArtNEBULAPlaPara*>& para_v)
{
  TGraph2DErrors* graph = new TGraph2DErrors;//staticを付けるとメモリの再確保がなくなり早い // memory leak

  double width = 12;
  double depth = 12;
  double xlength = 400;
  double ylength = 200;
  double zdepth = 1050;
  double zdepth_back = 200;
  double vl = 15.8;
  double treso = 0.35;

  for(int i=0; i<(int)data_v.size(); ++i){
    TDataNEBULA data = data_v[i];
    //    int id = data.id;
    //    const TArtNEBULAPlaPara* para = para_v[id-1];

    std::vector<double> pos = data.pos;
    
    graph->SetPoint(i, pos[0], pos[2], pos[1]);
    std::cout << i << " " << pos[0] << " " << pos[1] << " " << pos[2] << " " << data.time << std::endl;
    graph->SetPointError(i, width/sqrt(12), depth/sqrt(12), vl*treso/sqrt(2));

    graph->SetMarkerStyle(8);
    graph->SetMarkerSize(1);
    graph->GetXaxis()->SetLimits(-xlength/2., xlength/2.);
    graph->GetYaxis()->SetLimits(zdepth, zdepth+zdepth_back);
    //    graph->GetYaxis()->SetLimits(-1000, 2000);
    graph->SetMinimum(-ylength/2.);
    graph->SetMaximum(ylength/2.);
  }

  if(1){
    graph->Draw("PERR");
  }else{
    static bool init = true;
    if(init){
      graph->Draw("PERR");
      init = false;
    }else{
      graph->Draw("PERRSAME");
    }
  }

  for(int i=0; i<120; ++i){
    int id = i+1;
    const TArtNEBULAPlaPara* para = para_v[id-1];
    double x=para->GetDetPos(0);
    double y=para->GetDetPos(1);
    double z=para->GetDetPos(2);
    double dx=12;
    double dy=180;
    double dz=12;

    TMarker3DBox* box = new TMarker3DBox(x,z,y,dx/2,dz/2,dy/2,0,0);
    box->Draw("same");
  }

  gPad->GetCanvas()->Update();
}

double OneTrackFit(std::vector<TDataNEBULA>& data_v, std::vector<const TArtNEBULAPlaPara*>& para_v, double* parFit)
{
  if((int)data_v.size() < 3) return 10000;

  std::vector<double> pointx, pointy, pointz;
  std::vector<double> pointxe, pointye, pointze;
  for(int i=0; i<(int)data_v.size(); ++i){
    TDataNEBULA data = data_v[i];
    int id = data.id;
    const TArtNEBULAPlaPara* para = para_v[id-1];

    std::vector<double> pos = data.pos;
    if(para->GetSubLayer() != 0){ // NETU
      pointx.push_back(pos[0]);
      pointy.push_back(pos[1]);
      pointz.push_back(pos[2]);
      pointxe.push_back(12/sqrt(12));
      pointye.push_back(2);
      pointze.push_back(12/sqrt(12));
    }else{ // VETO
      pointx.push_back(pos[0]);
      pointy.push_back(pos[1]);
      pointz.push_back(pos[2]);
      pointxe.push_back(30/sqrt(12));
      pointye.push_back(10);
      pointze.push_back(1/sqrt(12)+0.5); // 0.5: measurement error
    }
  }

  TLinearFit::LinearFit(pointz, pointx, pointxe, parFit[0], parFit[1]);
  TLinearFit::LinearFit(pointz, pointy, pointye, parFit[2], parFit[3]);
  double aminx = TLinearFit::SD(pointz, pointx, parFit[0], parFit[1]);
  double aminy = TLinearFit::SD(pointz, pointy, parFit[2], parFit[3]);

  return sqrt(aminx*aminx+aminy*aminy);
}

#endif
