#ifndef __CINT__
#include "TH1.h"
#include "TF1.h"
#include "TFitResultPtr.h"
#include "TMath.h"

#include <vector>

#include "TArtAlias.hh"
#include "TArtHistManager.hh"
#endif

// look tutorials/fit/multifit.C for double gaussian fit

static const Double_t ratio = 0.044;

static Double_t qdiff;
static Double_t qdiff2;
static Double_t ratio2;

Double_t doublegaus(Double_t* x, Double_t* par);
Double_t stepgaus(Double_t* x, Double_t* par);

TFitResultPtr doublegausfit_time(int energy, double dsigma=4)
{
  std::vector<Double_t> par(6);

  if(energy == 200){
    // 194MeV
    qdiff = -3.432/(194)*66.3/(1.206*(1.206+1)); // -dE/E*TOF/2 (energy to time)
    qdiff2 = -1.93/(194)*66.3/(1.206*(1.206+1)); // -dE/E*TOF/2 (energy to time)
    ratio2 = 0.0814;
  }else if(energy == 250){
    // 246MeV
    qdiff = -3.432/(246)*(60.6)/(1.262*(1.262+1)); // -dE/E*TOF/2 (energy to time)
    qdiff2 = -1.67/(246)*(60.6)/(1.262*(1.262+1)); // -dE/E*TOF/2 (energy to time)
    ratio2 = 0.0486;
  }

  Double_t xmin = -1;
  Double_t xmax = 1;
  for(int i=0; i<4; ++i){ // fit range iteration
    //  TF1* dgaus = new TF1("dgaus", doublegaus, xmin, xmax, 3);
    TF1* dgaus = new TF1("dgaus", doublegaus, xmin, xmax, 5);
    dgaus->SetParameters(800, 0, 0.3, 0.824, 0.1); // 250 MeV のときはstep=0.749 MeVでいけるっぽい
    dgaus->SetParLimits(0, 10,8000); // counts coefficient
    dgaus->SetParLimits(1, -1, 1); // peak ns
    dgaus->SetParLimits(2, 0, 0.6); // resolution ns
    dgaus->SetParLimits(3, 0.6, 3); // gaus step start point meV
    //    dgaus->FixParameter(3, 0.853154); // gaus step start point meV // Fix @ 200 MeV
    dgaus->FixParameter(3, 0.73432); // gaus step start point meV // Fix @ 250 MeV
    dgaus->SetParLimits(4, 0, 0.5); // step gaus coefficient
    TFitResultPtr ptr = gH1->Fit(dgaus,"RS");

    // change range to +- 4 sigma
    Double_t mean = ptr->Parameter(1);
    Double_t sigma = ptr->Parameter(2);
    xmin = mean-sigma*dsigma;
    xmax = mean+sigma*dsigma;
    //    xmin = mean-sigma*4;
    //    xmax = mean+sigma*4;
    //    xmin = mean-sigma*3;
    //    xmax = mean+sigma*3;
  }

  TF1* gaus1 = new TF1("gaus1", "gausn", xmin, xmax);
  gaus1->SetParameters(ptr->Parameter(0),
		       ptr->Parameter(1),
		       ptr->Parameter(2));
  TF1* gaus2 = new TF1("gaus2", "gausn", xmin, xmax);
  gaus2->SetParameters(ptr->Parameter(0)*ratio,
		       ptr->Parameter(1)-qdiff,
		       ptr->Parameter(2));
  TF1* gaus3 = new TF1("gaus3", stepgaus, xmin, xmax, 4);
  gaus3->SetParameters(ptr->Parameter(0)*ptr->Parameter(4),
		       ptr->Parameter(1),
		       ptr->Parameter(2),
		       ptr->Parameter(3));
  TF1* gaus4 = new TF1("gaus4", "gausn", xmin, xmax);
  gaus4->SetParameters(ptr->Parameter(0)*ratio2,
		       ptr->Parameter(1)-qdiff2,
		       ptr->Parameter(2));

  TF1* gaus14 = new TF1("gaus14", "gausn(0)+gausn(3)", xmin, xmax); 
  gaus14->SetParameters(ptr->Parameter(0),
		       ptr->Parameter(1),
		       ptr->Parameter(2),
		       ptr->Parameter(0)*ratio2,
		       ptr->Parameter(1)-qdiff2,
		       ptr->Parameter(2));

//  dgaus->SetLineColor(1);
//  gaus1->SetLineColor(2);
//  gaus2->SetLineColor(3);
//  gaus3->SetLineColor(4);
//  gaus4->SetLineColor(5);

  dgaus->SetLineColor(1);
  gaus14->SetLineColor(2);
  gaus2->SetLineColor(kOrange+2);
  gaus3->SetLineColor(4);

  gH1->Draw("COLZH");

//  dgaus->Draw("same");
//  gaus1->Draw("same");
//  gaus2->Draw("same");
//  gaus3->Draw("same");
//  gaus4->Draw("same");

  gaus3->Draw("same");
  gaus2->Draw("same");
  dgaus->Draw("same");
  gaus14->Draw("same");

  gROOT->ProcessLine(".L macros/print_func.C");
  print_func(dgaus,1000,gH1->GetXaxis()->GetXmin(),gH1->GetXaxis()->GetXmax(), "neutron_time_total.txt");
  print_func(gaus14,1000,gH1->GetXaxis()->GetXmin(),gH1->GetXaxis()->GetXmax(),"neutron_time_7li.txt");
  print_func(gaus2,1000,gH1->GetXaxis()->GetXmin(),gH1->GetXaxis()->GetXmax(), "neutron_time_6li.txt");
  print_func(gaus3,1000,gH1->GetXaxis()->GetXmin(),gH1->GetXaxis()->GetXmax(), "neutron_time_high.txt");

  return ptr;
}

Double_t doublegaus(Double_t* x, Double_t* par)
{
  Double_t g1 = par[0]*TMath::Gaus(x[0], par[1], par[2], kTRUE);
  Double_t g2 = par[0]*ratio*TMath::Gaus(x[0], par[1]-qdiff, par[2], kTRUE);
  Double_t g3 = par[4]*stepgaus(x, par);
  Double_t g4 = par[0]*ratio2*TMath::Gaus(x[0], par[1]-qdiff2, par[2], kTRUE);  

  return g1+g2+g3+g4;
}

Double_t stepgaus(Double_t* x, Double_t* par)
{
  Double_t peak = par[1] + par[3];
  if(peak > x[0]){
    return par[0]*TMath::Gaus(x[0], peak, par[2], kTRUE);
  }else{
    return par[0]*TMath::Gaus(peak, peak, par[2], kTRUE);
  }
}
