
#include <iostream>
#include <sstream>
#include <fstream>
#include <vector>
#include <string>

#include "TGraphErrors.h"
#include "TCanvas.h"
#include "TPad.h"
#include "TAxis.h"
#include "TF1.h"
#include "TFitResultPtr.h"

#include "TArtAlias.hh"

void hpc_graph();
TF1* CreateLinear();
Double_t Linear(Double_t *x, Double_t *par);

void hpc_graph()
{
  std::vector<double> x[144];
  std::vector<double> xerr[144];
  std::vector<double> y[144];
  std::vector<double> yerr[144];
  {
    std::vector<int> idpeak1, idpeak2;
    for(int i=1; i<5; ++i){
      idpeak1.push_back(i);
      idpeak2.push_back(i+4);
    }
    for(int i=1; i<4; ++i){
      idpeak1.push_back(i);
      idpeak2.push_back(i+5);
    }
//    for(int i=2; i<5; ++i){
//      idpeak1.push_back(i);
//      idpeak2.push_back(i+3);
//    }
    for(int i=3; i<5; ++i){
      idpeak1.push_back(i);
      idpeak2.push_back(i+2);
    }
  
    std::vector<double> hpc_posy(9);
    hpc_posy[1]=78.5; // ホルダー下部の位置の気がする
    hpc_posy[2]=38.5;
    hpc_posy[3]=-0.5;
    hpc_posy[4]=-41.5;
    hpc_posy[5]=51.5;
    hpc_posy[6]=11.5;
    hpc_posy[7]=-28.5;
    hpc_posy[8]=-99.5;

    for(int i=0; i<(int)idpeak1.size(); ++i){
      std::ostringstream oss;
      oss << "txt/pos/pos" << idpeak1[i] << idpeak2[i] << ".txt";
      std::ifstream ifs(oss.str().c_str());
      if(!ifs){
	std::cout << "cannot open file:" << oss.str().c_str() << std::endl;
	return;
      }
      std::cout << "open " << oss.str().c_str() << std::endl;

      double hpcy1 = hpc_posy[idpeak1[i]] + 8.0; // 下部からHPC中心までの距離を足す。
      double hpcy2 = hpc_posy[idpeak2[i]] + 8.0; // 下部からHPC中心までの距離を足す。
      double hpcz = 40.5; // NEBULAの中心からの距離
      double neutz = 6.6; // NEBULAの中心からの距離
      double vetoz1 = 34.0; //  NEBULAの中心からの距離
      double vetoz2 = 32.0; //  NEBULAの中心からの距離
      double neuty1 = (hpcz-neutz)/(2*hpcz)*(hpcy2-hpcy1)+hpcy1;
      double neuty2 = (hpcz+neutz)/(2*hpcz)*(hpcy2-hpcy1)+hpcy1;
      double vetoy1 = (hpcz-vetoz1)/(2*hpcz)*(hpcy2-hpcy1)+hpcy1; 
      double vetoy2 = (hpcz-vetoz2)/(2*hpcz)*(hpcy2-hpcy1)+hpcy1; 

      //      std::cout << i << " " << idpeak1[i] << " " << idpeak2[i] << " " << neuty1 << " " << neuty2 << std::endl;

      while(1){
	std::string str;
	std::getline(ifs, str);
	if(ifs.eof()) break;
	std::istringstream iss(str);

	std::vector<double> x_v;
	double xtemp;
	while(iss >> xtemp) x_v.push_back(xtemp);

	int index = x_v[0]-1;
	int id = x_v[0];
	x[index].push_back(x_v[3]);
	xerr[index].push_back(x_v[4]);
	if(id > 120){ // VETO
	  if(id % 2 == 1){
	    y[index].push_back(vetoy1);
	  }else{
	    y[index].push_back(vetoy2);
	  }
	}else{ // NEUT
	  if((id-1) / 30 % 2 == 0){
	    y[index].push_back(neuty1);
	  }else{
	    y[index].push_back(neuty2);
	  }
	}
	yerr[index].push_back(0.1);
	//	std::cout << x[index].back() << " " << xerr[index].back() << " " << y[index].back() << " " << yerr[index].back() << std::endl;
      }
    }
  }

  std::ofstream ofs("pos_graph.txt");
  if(!ofs){
    std::cout << "cannot open file" << std::endl;
    return;
  }

  for(int i=0; i<144; ++i){
//    TGraphErrors* graph = new TGraphErrors(y[i].size(),
//					   &y[i][0], &x[i][0],
//					   &yerr[i][0], &xerr[i][0]);
    TGraphErrors* graph = new TGraphErrors(x[i].size(),
					   &x[i][0], &y[i][0],
					   &xerr[i][0], &yerr[i][0]);
    //    graph->GetXaxis()->SetRangeUser(0,40);
    //    graph->GetXaxis()->SetLimits(-5,35);
    //    graph->SetMinimum(-5);
    //    graph->SetMaximum(35);
    graph->Draw("AP");
    TFitResultPtr frp = graph->Fit("pol1","QS","");
    //    TFitResultPtr frp = graph->Fit(func,"QS","",0,40);
    ofs << i+1 << " " << frp->Parameter(0) << " " << frp->ParError(0) << " "
	<< frp->Parameter(1) << " " << frp->ParError(1) << " " 
	<< frp->Chi2() << std::endl;
    
    gPad->GetCanvas()->Update();
    if(!wait()) break;
  }
}

TF1* CreateLinear()
{
  return new TF1("linear", Linear, 0, 40, 1);
}

Double_t Linear(Double_t *x, Double_t *par)
{
  return par[0]*x[0];
}
