
#include "TFile.h"
#include "TTree.h"
#include "TBranch.h"
#include "TH1.h"
#include "TH2D.h"
#include "TAxis.h"
#include "TRandom.h"
#include "TFitResult.h"
#include "TFitResultPtr.h"
#include "TPad.h"
#include "TCanvas.h"

#include <iostream>
#include <vector>
#include <sstream>
#include <fstream>

#include "TArtSAMURAIParameters.hh"
#include "TArtNEBULAHPCPara.hh"
#include "TArtNEBULAPlaPara.hh"

#include "TArtAlias.hh"
#include "TArtHistManager.hh"

void nebula_tree_hpc_all(const char* filename, int nentries = -1);
void nebula_tree_hpc(const char* filename, int nentries = -1, int idpeak1 = 1, int idpeak2 = 5, bool cleareflag=false);

void nebula_tree_hpc_all(const char* filename, int nentries)
{
  std::vector<int> idpeak1, idpeak2;
  for(int i=1; i<5; ++i){
    idpeak1.push_back(i);
    idpeak2.push_back(i+4);
  }
  for(int i=1; i<4; ++i){
    idpeak1.push_back(i);
    idpeak2.push_back(i+5);
  }
//  for(int i=2; i<5; ++i){ // few events
//    idpeak1.push_back(i);
//    idpeak2.push_back(i+3);
//  }
  for(int i=3; i<5; ++i){
    idpeak1.push_back(i);
    idpeak2.push_back(i+2);
  }
  
  for(int i=0; i<(int)idpeak1.size(); ++i){
    nebula_tree_hpc(filename, nentries, idpeak1[i], idpeak2[i], true);
  }
}

void nebula_tree_hpc(const char* filename, int nentries, int idpeak1, int idpeak2, bool clearflag)
{
  std::ostringstream oss;
  oss << "pos" << idpeak1 << idpeak2 << ".txt";
  std::ofstream ofs(oss.str().c_str());

  TFile* file = TFile::Open(filename);
  if(file->IsZombie()) return;

  TTree* tree;
  file->GetObject("tree",tree);
  tree->Print();

  std::vector<Double_t>* hpcid_v = new std::vector<Double_t>;
  std::vector<Double_t>* hpctraw_v = new std::vector<Double_t>;
  std::vector<Double_t>* id_v = new std::vector<Double_t>;
  std::vector<Double_t>* quraw_v = new std::vector<Double_t>;
  std::vector<Double_t>* qdraw_v = new std::vector<Double_t>;
  std::vector<Double_t>* turaw_v = new std::vector<Double_t>;
  std::vector<Double_t>* tdraw_v = new std::vector<Double_t>;
  TBranch *hpcid_b, *hpctraw_b;
  TBranch *id_b, *quraw_b, *qdraw_b, *turaw_b, *tdraw_b;
  if(tree->SetBranchAddress("hpcid",&hpcid_v,&hpcid_b) < 0) return;
  if(tree->SetBranchAddress("hpctraw",&hpctraw_v,&hpctraw_b) < 0) return;
  if(tree->SetBranchAddress("id",&id_v,&id_b) < 0) return;
  if(tree->SetBranchAddress("quraw",&quraw_v,&quraw_b) < 0) return;
  if(tree->SetBranchAddress("qdraw",&qdraw_v,&qdraw_b) < 0) return;
  if(tree->SetBranchAddress("turaw",&turaw_v,&turaw_b) < 0) return;
  if(tree->SetBranchAddress("tdraw",&tdraw_v,&tdraw_b) < 0) return;

  TH1* hidy = new TH2D("hidy", "id vs y", 144, 0.5,144.5, 200,-100,100);
  std::vector<TH1*> hdtslw(145,(TH1*)0);
  std::vector<TH1*> hdtslwqave(145,(TH1*)0);
  for(int i=0; i<145; ++i){
    {
      std::ostringstream name,title;
      name << "hdtslw" << i;
      title << "dtslw id" << i;
      if(i == 0){
	hdtslw[i] = new TH2D(name.str().c_str(), title.str().c_str(), 144, 0.5, 144.5, 200,-20,20);
      }else{
	hdtslw[i] = new TH1D(name.str().c_str(), title.str().c_str(), 200,-20,20);
      }
    }
    {
      std::ostringstream name,title;
      name << "hdtslwqave" << i;
      title << "dtslw vs qave id" << i;
      hdtslwqave[i] = new TH2D(name.str().c_str(), title.str().c_str(), 200,-20,20, 400,0,50);
    }
  }

  TArtSAMURAIParameters *samuraiparameters = TArtSAMURAIParameters::Instance();
  samuraiparameters->LoadParameter((char*)"db/NEBULAHPC.xml");
  samuraiparameters->LoadParameter((char*)"db/NEBULA.xml");

  std::vector<const TArtNEBULAPlaPara*> para_v;
  for(int i=0; i<144; ++i){
    int id=i+1;
    const TArtNEBULAPlaPara* para = samuraiparameters->FindNEBULAPlaPara(id);
    para_v.push_back(para);
  }

  if(nentries < 0) nentries = tree->GetEntriesFast();
  for(int event=0; event<nentries; ++event) {
    if((event+1)%10000 == 0) std::cout << "\r converted events:" << event+1 << std::flush;

    Long64_t entry = tree->LoadTree(event);
    tree->GetEntry(entry);

    // HPC
    bool hpcidflag[16+1];
    for(int j=0; j<16+1; ++j) hpcidflag[j] = false;
    for(int j=0; j<(int)hpcid_v->size(); ++j){
      int hpcid = (int)(*hpcid_v)[j];
      hpcidflag[hpcid] = true;
    }

    // NEBULA
    for(int j=0; j<(int)id_v->size(); ++j){
      int id = (int)(*id_v)[j];
      int idpeak3 = idpeak1+8;
      int idpeak4 = idpeak2+8;
      if((id >= 1 && id <= 60) ||
	 (id >= 121 && id <= 132)){
	if(!((hpcidflag[idpeak1] && hpcidflag[idpeak2]))) continue;
//	if(!((hpcidflag[1] && hpcidflag[5]) ||
//	     (hpcidflag[2] && hpcidflag[6]) ||
//	     (hpcidflag[3] && hpcidflag[7]) ||
//	     (hpcidflag[4] && hpcidflag[8]))) continue;
      }else{
	if(!((hpcidflag[idpeak3] && hpcidflag[idpeak4]))) continue;
//	if(!((hpcidflag[1+8] && hpcidflag[5+8]) ||
//	     (hpcidflag[2+8] && hpcidflag[6+8]) ||
//	     (hpcidflag[3+8] && hpcidflag[7+8]) ||
//	     (hpcidflag[4+8] && hpcidflag[8+8]))) continue;
      }	

      double quraw = (*quraw_v)[j];
      double qdraw = (*qdraw_v)[j];
      double turaw = (*turaw_v)[j];
      double tdraw = (*tdraw_v)[j];
      const TArtNEBULAPlaPara* para = para_v[id-1];

      if(turaw < 0 || turaw > 4094 ||
	 tdraw < 0 || tdraw > 4094 ||
	 quraw < 0 || quraw > 4094 ||
	 qdraw < 0 || qdraw > 4094){
	continue;
      }

      Double_t quped = quraw - para->GetQUPed();
      Double_t qdped = qdraw - para->GetQDPed();
      Double_t qucal = quped * para->GetQUCal();
      Double_t qdcal = qdped * para->GetQDCal();
      //      Double_t qaveped = sqrt(quped*qdped);
      Double_t qavecal = para->GetQAveCal()*sqrt(qucal*qdcal);

      if(para->GetSubLayer() != 0){ // NEUT
	if(qavecal < 20 || qavecal > 40){
	  continue;
	}
      }else{ // VETO
	if(qavecal < 1 || qavecal > 8){
	  continue;
	}
      }
	
      Double_t tucal = turaw * para->GetTUCal() + para->GetTUOff();
      Double_t tdcal = tdraw * para->GetTDCal() + para->GetTDOff();
      Double_t tuslw = tucal;
      Double_t tdslw = tdcal;
      Double_t logquped = log(quped);
      Double_t logqdped = log(qdped);
      Double_t logquped2 = logquped*logquped;
      Double_t logqdped2 = logqdped*logqdped;
      Double_t logquped4 = logquped2*logquped2;
      Double_t logqdped4 = logqdped2*logqdped2;
      Double_t a,b,c,d,e,f;
      a = 0;
      b = para->GetTUSlwLog(0);
      c = para->GetTUSlwLog(1);
      d = para->GetTUSlwLog(2);
      e = para->GetTUSlwLog(3);
      f = para->GetTUSlwLog(4);
      tuslw -= (a +
		b*logquped +
		c*logquped2 +
		d*logquped2*logquped +
		e*logquped4 +
		f*logquped4*logquped);
      tdslw -= (a +
		b*logqdped +
		c*logqdped2 +
		d*logqdped2*logqdped +
		e*logqdped4 +
		f*logqdped4*logqdped);
      //      Double_t dtraw = tdraw - turaw;
      //      Double_t dtcal = tdcal - tucal;
      Double_t dtslw = tdslw - tuslw;
      //      Double_t tavecal = (tucal + tdcal)/2 + para->GetTAveOff();
      //      Double_t taveslw = (tuslw + tdslw)/2 + para->GetTAveOff();
      //      Double_t poscal = dtcal * para->GetDTCal() + para->GetDTOff();
      Double_t posslw = dtslw * para->GetDTCal() + para->GetDTOff();

      std::vector<Double_t> pos(3);
      if(para->GetSubLayer() != 0){ // NEUT
	//	pos[0] = para->GetDetPos(0) + gRandom->Uniform(-6,6);
	pos[0] = para->GetDetPos(0);
      }else{ // VETO
	//	pos[0] = para->GetDetPos(0) + gRandom->Uniform(-16,16);
	pos[0] = para->GetDetPos(0);
      }
      pos[1] = posslw + para->GetDetPos(1);
      pos[2] = para->GetDetPos(2);

      hidy->Fill(id, pos[1]);
      hdtslw[0]->Fill(id, dtslw);
      hdtslwqave[0]->Fill(dtslw, qavecal);

      hdtslw[id]->Fill(dtslw);
      hdtslwqave[id]->Fill(dtslw, qavecal);
    }
  }
  std::cout << std::endl;

  for(int i=1; i<145; ++i){
    TH1* hist = hdtslw[i];
    int maxbin = hist->GetMaximumBin();
    Double_t maxx = hist->GetXaxis()->GetBinCenter(maxbin);
    TFitResultPtr frp = hist->Fit("gaus", "QS", "", maxx-0.5, maxx+0.5);
    //    if(!frp.Get()) continue;
    for(int j=0; j<4; ++j){
      Double_t sigma = frp->Parameter(2);
      frp = hist->Fit("gaus", "QS", "", maxx-2*sigma, maxx+2*sigma);
    }

    ofs << i << " "
	<< frp->Parameter(0) << " " << frp->ParError(0) << " " 
	<< frp->Parameter(1) << " " << frp->ParError(1) << " " 
	<< frp->Parameter(2) << " " << frp->ParError(2) << std::endl;
    
    gPad->GetCanvas()->Update();
    //    if(!wait()) break;
  }

  tree->ResetBranchAddresses();

  if(clearflag){
    for(int i=0; i<144+1; ++i){
      delete hdtslw[i];
    }
    delete hpcid_v;
    delete hpctraw_v;
    delete id_v;
    delete quraw_v;
    delete qdraw_v;
    delete turaw_v;
    delete tdraw_v;

    file->Close();
  }
}
