
#include "TFile.h"
#include "TTree.h"
#include "TBranch.h"
#include "TH1.h"
#include "TH2D.h"
#include "TH3D.h"
#include "TRandom.h"

#include <iostream>
#include <vector>
#include <sstream>
#include <cmath>

#include "TArtBigRIPSParameters.hh"
#include "TArtPlasticPara.hh"

#include "TArtSAMURAIParameters.hh"
#include "TArtNEBULAPlaPara.hh"

#include "TDataNEBULA.hh"

typedef TDataNEBULA data_t;

static double neut_thr = 6;

void SetNeutThr(double thr)
{
  neut_thr = thr;
}

void nebula_tree_neutron(const char* filename, int nentries = -1);

void nebula_tree_neutron(const char* filename, int nentries)
{
  TFile* file = TFile::Open(filename);
  if(!file || file->IsZombie()) return;

  TTree* tree;
  file->GetObject("tree",tree);
  tree->Print();

  std::vector<Double_t>* id_v = new std::vector<Double_t>;
  std::vector<Double_t>* quraw_v = new std::vector<Double_t>;
  std::vector<Double_t>* qdraw_v = new std::vector<Double_t>;
  std::vector<Double_t>* turaw_v = new std::vector<Double_t>;
  std::vector<Double_t>* tdraw_v = new std::vector<Double_t>;
  TBranch *id_b, *quraw_b, *qdraw_b, *turaw_b, *tdraw_b;
  if(tree->SetBranchAddress("id",&id_v,&id_b) < 0) return;
  if(tree->SetBranchAddress("quraw",&quraw_v,&quraw_b) < 0) return;
  if(tree->SetBranchAddress("qdraw",&qdraw_v,&qdraw_b) < 0) return;
  if(tree->SetBranchAddress("turaw",&turaw_v,&turaw_b) < 0) return;
  if(tree->SetBranchAddress("tdraw",&tdraw_v,&tdraw_b) < 0) return;

  std::vector<Double_t>* plaid_v = new std::vector<Double_t>;
  std::vector<Double_t>* plaqlraw_v = new std::vector<Double_t>;
  std::vector<Double_t>* plaqrraw_v = new std::vector<Double_t>;
  std::vector<Double_t>* platlraw_v = new std::vector<Double_t>;
  std::vector<Double_t>* platrraw_v = new std::vector<Double_t>;
  TBranch *plaid_b, *plaqlraw_b, *plaqrraw_b, *platlraw_b, *platrraw_b;
  if(tree->SetBranchAddress("plaid",   &plaid_v,   &plaid_b) < 0) return;
  if(tree->SetBranchAddress("plaqlraw",&plaqlraw_v, &plaqlraw_b) < 0) return;
  if(tree->SetBranchAddress("plaqrraw",&plaqrraw_v, &plaqrraw_b) < 0) return;
  if(tree->SetBranchAddress("platlraw",&platlraw_v, &platlraw_b) < 0) return;
  if(tree->SetBranchAddress("platrraw",&platrraw_v, &platrraw_b) < 0) return;

  std::vector<Double_t>* coin_v = new std::vector<Double_t>;
  TBranch *coin_b;
  if(tree->SetBranchAddress("coin",&coin_v,&coin_b) < 0) return;

  Double_t t0;
  TBranch *t0_b;
  if(tree->SetBranchAddress("t0",&t0,&t0_b) < 0) return;

  TH1* hcoin = new TH1D("hcoin", "coincidence register", 16, 0.5, 16.5);
  TH1* hcoingated = new TH1D("hcoingated", "coincidence register", 16, 0.5, 16.5);

  TH1* ht0 = new TH1D("ht0", "t0", 800, 40, 80);
  TH1* hsbttq = new TH2D("hsbttq", "sbt t vs q", 200, 40, 70, 200, 0,2000);
  TH1* hsbtdtq = new TH2D("hsbtdtq", "sbt dt vs q", 200, -10, 10, 200, 0,2000);
  TH1* hsbtdtt = new TH2D("hsbtdtt", "sbt dt vs t", 200, -10, 10, 200,40,70);

  TH1* henergy = new TH1D("henergy", "neutron energy", 1200, 0, 300);
  TH1* henergytheta = new TH2D("henergytheta", "neutron energy vs theta", 1000, 145, 245, 2000, 0, 0.2);
  TH1* htimetheta = new TH2D("htimetheta", "neutron time vs theta", 1000, -5, 5, 2000, 0, 0.2);
  TH1* hidqavecal = new TH2D("hidqavecal", "id vs qavecal", 144, 0.5, 144.5, 300, 0, 150);
  TH1* hidtaveslwt0 = new TH2D("hidtaveslwt0", "id vs taveslwt0", 144, 0.5, 144.5, 300, 0, 150);
  TH1* hqavetave = new TH2D("hqavetave", "qave vs tave", 200, 0, 4000, 400, -5, 5);
  TH1* htaveqave = new TH2D("htaveqave", "tave vs qave", 1400, 0, 140, 320, 0, 160);
  TH1* hsbtqtave = new TH2D("hsbtqtave", "sbtq vs tave", 400, 0, 2000, 400, -5, 5);
  TH1* hsbtttave = new TH2D("hsbtttave", "sbtt vs tave", 400, 40, 70, 400, -5, 5);

  TH1* henergytheta_v[3][3];
  henergytheta_v[1][1] = new TH2D("henergytheta1", "neutron energy vs theta 1", 500, 145, 245, 1000, 0, 0.2);
  henergytheta_v[1][2] = new TH2D("henergytheta2", "neutron energy vs theta 2", 500, 145, 245, 1000, 0, 0.2);
  henergytheta_v[2][1] = new TH2D("henergytheta3", "neutron energy vs theta 3", 500, 145, 245, 1000, 0, 0.2);
  henergytheta_v[2][2] = new TH2D("henergytheta4", "neutron energy vs theta 4", 500, 145, 245, 1000, 0, 0.2);

  TArtBigRIPSParameters *bigripsparameters = TArtBigRIPSParameters::Instance();
  bigripsparameters->LoadParameter((char*)"db/BigRIPSPlastic.xml");
  TArtSAMURAIParameters *samuraiparameters = TArtSAMURAIParameters::Instance();
  samuraiparameters->LoadParameter((char*)"db/NEBULA.xml");

  std::vector<const TArtNEBULAPlaPara*> para_v;
  for(int i=0; i<144; ++i){
    int id=i+1;
    const TArtNEBULAPlaPara* para = samuraiparameters->FindNEBULAPlaPara(id);
    para_v.push_back(para);
  }

  std::vector<const TArtPlasticPara*> plapara_v;
  for(int i=0; i<7; ++i){
    int id=i+1;
    const TArtPlasticPara* plapara = bigripsparameters->FindPlasticPara(id);
    plapara_v.push_back(plapara);
  }

  if(nentries < 0) nentries = tree->GetEntriesFast();
  std::vector<data_t> data_v;
  for(int event=0; event<nentries; ++event) {
    if((event+1)%10000 == 0) std::cout << "\r converted events:" << event+1 << std::flush;

    Long64_t entry = tree->LoadTree(event);

    // coin
    coin_b->GetEntry(entry);
    std::vector<int> coin(16, 0);
    for(int j=0; j<(int)coin_v->size(); ++j){    
      int coin_id = (int)(*coin_v)[j];
      hcoin->Fill(coin_id);
      coin[coin_id] = 1;
    }

    // SBT
    plaid_b->GetEntry(entry);
    plaqlraw_b->GetEntry(entry);
    plaqrraw_b->GetEntry(entry);
    platlraw_b->GetEntry(entry);
    platrraw_b->GetEntry(entry);
    std::vector<double> qave(6, 4095), tave(6, 4095);
    for(int j=0; j<(int)plaid_v->size(); ++j){
      int id = (int)(*plaid_v)[j];
      if(!(id == 4 || id == 5)) continue;
      double qlraw = (*plaqlraw_v)[j];
      double qrraw = (*plaqrraw_v)[j];
      double tlraw = (*platlraw_v)[j];
      double trraw = (*platrraw_v)[j];
      const TArtPlasticPara* para = plapara_v[id-1];

      if(!(tlraw >= 0 && tlraw < 4095 &&
	   trraw >= 0 && trraw < 4095)){
	static int errorcounter=0;
	++errorcounter;
	std::cout << "\n sbt time information lost #" << errorcounter << std::endl;
	continue;
      }

      double qlped = qlraw-para->GetQPedLeft();
      double qrped = qrraw-para->GetQPedRight();
      qave[id] = sqrt(qlped*qrped);
      double tlcal = tlraw*para->GetTCalLeft()+tlraw*tlraw*para->GetTCalLeft2();
      double trcal = trraw*para->GetTCalRight()+trraw*trraw*para->GetTCalRight2();
      double tlslw = tlcal+para->GetTLSlewA()/sqrt(qlped);
      double trslw = trcal+para->GetTRSlewA()/sqrt(qrped);
      tave[id] = (tlslw+trslw)/2;
    }
    double sbtt = (tave[4]+tave[5])/2;
    double sbtdt = tave[4]-tave[5];
    double sbtq = (qave[4]+qave[5])/2;
    if(!(sbtq < 403-62*2)) continue;
    //    if(!(sbtt > 57.83-5*0.28 && sbtt < 57.83+5*0.28)) continue;
    if(!(sbtdt > 2.74-5*0.17 && sbtdt < 2.74+5*0.17)) continue ;

    for(int j=0; j<(int)coin_v->size(); ++j){    
      int coin_id = (int)(*coin_v)[j];
      hcoingated->Fill(coin_id);
    }

    if(!coin[2]) continue;

    hsbttq->Fill(sbtt, sbtq);
    hsbtdtq->Fill(sbtdt, sbtq);
    hsbtdtt->Fill(sbtdt, sbtt);

    // t0
    //    t0_b->GetEntry(entry);
    t0 = sbtt;

    // NEBULA
    id_b->GetEntry(entry);
    quraw_b->GetEntry(entry);
    qdraw_b->GetEntry(entry);
    turaw_b->GetEntry(entry);
    tdraw_b->GetEntry(entry);
    data_v.clear();
    for(int j=0; j<(int)id_v->size(); ++j){
      int id = (int)(*id_v)[j];
      double quraw = (*quraw_v)[j];
      double qdraw = (*qdraw_v)[j];
      double turaw = (*turaw_v)[j];
      double tdraw = (*tdraw_v)[j];
      const TArtNEBULAPlaPara* para = para_v[id-1];

      if(!(turaw >= 0 && turaw < 4095 &&
	   tdraw >= 0 && tdraw < 4095 &&
	   quraw >= 0 && quraw < 4095 &&
	   qdraw >= 0 && qdraw < 4095)){
	continue;
      }

      Double_t quped = quraw - para->GetQUPed();
      Double_t qdped = qdraw - para->GetQDPed();
      if(!(quped > 0 && qdped > 0)) continue;
      Double_t qucal = quped * para->GetQUCal();
      Double_t qdcal = qdped * para->GetQDCal();
      Double_t qaveped = sqrt(quped*qdped);
      Double_t qavecal = para->GetQAveCal()*sqrt(qucal*qdcal);

      Double_t tucal = turaw * para->GetTUCal() + para->GetTUOff();
      Double_t tdcal = tdraw * para->GetTDCal() + para->GetTDOff();
      Double_t tuslw = tucal;
      Double_t tdslw = tdcal;
      Double_t logquped = log(quped);
      Double_t logqdped = log(qdped);
      Double_t logquped2 = logquped*logquped;
      Double_t logqdped2 = logqdped*logqdped;
      Double_t logquped4 = logquped2*logquped2;
      Double_t logqdped4 = logqdped2*logqdped2;

      Double_t a,b,c,d,e,f;
      a = 0;
      b = para->GetTUSlwLog(0);
      c = para->GetTUSlwLog(1);
      d = para->GetTUSlwLog(2);
      e = para->GetTUSlwLog(3);
      f = para->GetTUSlwLog(4);
	
      tuslw -= (a +
  		b*logquped +
		c*logquped2 +
		d*logquped2*logquped +
		e*logquped4 +
		f*logquped4*logquped);
      tdslw -= (a +
		b*logqdped +
		c*logqdped2 +
		d*logqdped2*logqdped +
		e*logqdped4 +
		f*logqdped4*logqdped);

      Double_t dtraw = tdraw - turaw;
      Double_t dtcal = tdcal - tucal;
      Double_t dtslw = tdslw - tuslw;
      //      Double_t taveraw = (turaw + tdraw)/2.;
      Double_t tavecal = (tucal + tdcal)/2 + para->GetTAveOff();
      Double_t taveslw = (tuslw + tdslw)/2 + para->GetTAveOff();
      Double_t poscal = dtcal * para->GetDTCal() + para->GetDTOff();
      Double_t posslw = dtslw * para->GetDTCal() + para->GetDTOff();

      Double_t tucalt0 = tucal - t0;
      Double_t tdcalt0 = tdcal - t0;
      Double_t tuslwt0 = tuslw - t0;
      Double_t tdslwt0 = tdslw - t0;

      std::vector<Double_t> pos(3);
      if(para->GetSubLayer() != 0){ // NEUT
	//	pos[0] = para->GetDetPos(0) + gRandom->Uniform(-6,6);
	pos[0] = para->GetDetPos(0);
      }else{ // VETO
	//	pos[0] = para->GetDetPos(0) + gRandom->Uniform(-16,16);
	pos[0] = para->GetDetPos(0);
      }
      pos[1] = posslw + para->GetDetPos(1);
      pos[2] = para->GetDetPos(2);

      qavecal /= 1+pos[1]*pos[1]*para->GetQAveCalAtt();

      Double_t flightlength = sqrt(pos[0]*pos[0]+pos[1]*pos[1]+pos[2]*pos[2]);
      Double_t flightangle = atan(sqrt(pos[0]*pos[0]+pos[1]*pos[1])/pos[2]);
      Double_t flightbeta = 0.55980;
      Double_t flightbetacor = 0.297595*cos(0.857825*flightangle)+0.702498+0.0486969*flightangle*flightangle;
      flightbeta *= flightbetacor;

      //      Double_t df13target = 228; // f13-target distance. +-1cm 程度のはず
      Double_t TOFF13TGT = 13.49;
      Double_t tavecalt0 = (tucalt0+tdcalt0)/2 + para->GetTAveOff() - TOFF13TGT - 5.361;
      Double_t taveslwt0 = (tuslwt0+tdslwt0)/2 + para->GetTAveOff() - TOFF13TGT - 5.361;
      Double_t tavecalt0flcor = (tucalt0+tdcalt0)/2 + para->GetTAveOff() - TOFF13TGT - flightlength/(flightbeta*29.979) - 5.361;
      Double_t taveslwt0flcor = (tuslwt0+tdslwt0)/2 + para->GetTAveOff() - TOFF13TGT - flightlength/(flightbeta*29.979) - 5.361;

      Double_t time = taveslwt0;
      std::vector<Double_t> beta(3), p(3);
      for(Int_t i=0; i<3; ++i){
	beta[i] = pos[i]/time/29.979;
      }
      Double_t betaabs = sqrt(beta[0]*beta[0]+beta[1]*beta[1]+beta[2]*beta[2]);
      Double_t gamma = 1/sqrt(1-betaabs*betaabs);
      for(Int_t i=0; i<3; ++i){
	p[i] = mneutron*gamma*beta[i];
      }
      Double_t pabs = sqrt(p[0]*p[0]+p[1]*p[1]+p[2]*p[2]);
      Double_t energy = sqrt(mneutron*mneutron + pabs*pabs) - mneutron;
      Double_t posx = pos[0]+ gRandom->Uniform(-6,6);
      Double_t theta = atan(sqrt(posx*posx+pos[1]*pos[1])/pos[2]);
      //      Double_t theta = atan(sqrt(pos[0]*pos[0]+pos[1]*pos[1])/pos[2]);
      Double_t phi = atan(pos[1]/posx);
      if(posx < 0 && pos[1] > 0){
	phi += M_PI;
      }else if(posx < 0 && pos[1] < 0){
	phi -= M_PI;
      }
      Double_t flightenergy = 194.158;
      Double_t flightenergycor = 0.294442*cos(1.18132*theta)+0.705526+0.0467828*theta*theta;
      energy += flightenergy*(1-flightenergycor);

      data_t data;
      data.id = id;
      data.quraw = quraw;
      data.qdraw = qdraw;
      data.turaw = turaw;
      data.tdraw = tdraw;
      data.quped = quped;
      data.qdped = qdped;
      data.qucal = qucal;
      data.qdcal = qdcal;
      data.qaveped = qaveped;
      data.qavecal = qavecal;
      data.tucal = tucal;
      data.tdcal = tdcal;
      data.tuslw = tuslw;
      data.tdslw = tdslw;
      data.tavecal = tavecal;
      data.taveslw = taveslw;
      data.logquped = logquped;
      data.logqdped = logqdped;
      data.dtraw = dtraw;
      data.dtcal = dtcal;
      data.dtslw = dtslw;
      data.poscal = poscal;
      data.posslw = posslw;
      data.tucalt0 = tucalt0;
      data.tdcalt0 = tdcalt0;
      data.tuslwt0 = tuslwt0;
      data.tdslwt0 = tdslwt0;
      data.tavecalt0 = tavecalt0;
      data.taveslwt0 = taveslwt0;
      data.tavecalt0flcor = tavecalt0flcor;
      data.taveslwt0flcor = taveslwt0flcor;
      data.pos = pos;
      data.flightlength = flightlength;

      data.time = time;
      data.beta = beta;
      data.betaabs = betaabs;
      data.gamma = gamma;
      data.p = p;
      data.pabs = pabs;
      data.energy = energy;
      data.theta = theta;
      data.phi = phi;

      data_v.push_back(data);
    }

    //    threshold(data_v, para_v, 6, 1);
    //    threshold(data_v, para_v, 2, 1);
    threshold(data_v, para_v, neut_thr, 1);

    if(veto(data_v, para_v, 1)) continue;
    //    if(veto(data_v, para_v, 2)) continue;
    //    if(vetolocal(data_v, para_v)) continue;

    for(int i=0; i<(int)data_v.size(); ++i){
      if(data_v[i].taveslwt0flcor < -2){
	data_v.erase(data_v.begin()+i);
	--i;
      }
    }
    hitmintime(data_v, para_v);

    //    acceptance(data_v, para_v, 0.08);
    //    acceptance(data_v, para_v, 0.04);
    //    acceptance2(data_v, para_v, 0.04, 0.08);

    ht0->Fill(t0);

    for(int i=0; i<(int)data_v.size(); ++i){
      data_t data = data_v[i];
      int id = data.id;
      const TArtNEBULAPlaPara* para = para_v[id-1];

      double qaveped = data.qaveped;
      double qavecal = data.qavecal;
      double taveslwt0 = data.taveslwt0;
      double taveslwt0flcor = data.taveslwt0flcor;
      double energy = data.energy;
      double theta = data.theta;

      henergy->Fill(energy);
      henergytheta->Fill(energy, theta);
      henergytheta_v[para->GetLayer()][para->GetSubLayer()]->Fill(energy, theta);
      htimetheta->Fill(taveslwt0flcor, theta);
      hidqavecal->Fill(id,qavecal);
      hidtaveslwt0->Fill(id,taveslwt0);
      hqavetave->Fill(qaveped,taveslwt0flcor);
      htaveqave->Fill(taveslwt0,qavecal);

      if(qaveped > 500 && qaveped < 2000){
	hsbtqtave->Fill(sbtq, taveslwt0flcor);
	hsbtttave->Fill(sbtt, taveslwt0flcor);
      }
    }
  }
  std::cout << std::endl;

  tree->ResetBranchAddresses();
}
